@@ -1,10 +1,20 @@
 from __future__ import print_function, division
 
-from sympy.core import S, C, sympify, cacheit
+from sympy.core import S, sympify, cacheit
+from sympy.core.add import Add
 from sympy.core.function import Function, ArgumentIndexError, _coeff_isneg
 
 from sympy.functions.elementary.miscellaneous import sqrt
 
+from sympy.functions.elementary.exponential import exp, log
+from sympy.functions.combinatorial.factorials import factorial, RisingFactorial
+
+def _rewrite_hyperbolics_as_exp(expr):
+    expr = sympify(expr)
+    return expr.xreplace(dict([(h, h.rewrite(exp))
+        for h in expr.atoms(HyperbolicFunction)]))
+
+
 ###############################################################################
 ########################### HYPERBOLIC FUNCTIONS ##############################
 ###############################################################################
@@ -23,6 +33,39 @@ class HyperbolicFunction(Function):
     unbranched = True
 
 
+def _peeloff_ipi(arg):
+    """
+    Split ARG into two parts, a "rest" and a multiple of I*pi/2.
+    This assumes ARG to be an Add.
+    The multiple of I*pi returned in the second position is always a Rational.
+
+    Examples
+    ========
+
+    >>> from sympy.functions.elementary.hyperbolic import _peeloff_ipi as peel
+    >>> from sympy import pi, I
+    >>> from sympy.abc import x, y
+    >>> peel(x + I*pi/2)
+    (x, I*pi/2)
+    >>> peel(x + I*2*pi/3 + I*pi*y)
+    (x + I*pi*y + I*pi/6, I*pi/2)
+    """
+    for a in Add.make_args(arg):
+        if a == S.Pi*S.ImaginaryUnit:
+            K = S.One
+            break
+        elif a.is_Mul:
+            K, p = a.as_two_terms()
+            if p == S.Pi*S.ImaginaryUnit and K.is_Rational:
+                break
+    else:
+        return arg, S.Zero
+
+    m1 = (K % S.Half)*S.Pi*S.ImaginaryUnit
+    m2 = K*S.Pi*S.ImaginaryUnit - m1
+    return arg - m2, m2
+
+
 class sinh(HyperbolicFunction):
     r"""
     The hyperbolic sine function, `\frac{e^x - e^{-x}}{2}`.
@@ -52,6 +95,8 @@ class sinh(HyperbolicFunction):
 
     @classmethod
     def eval(cls, arg):
+        from sympy import sin
+
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -72,11 +117,16 @@ class sinh(HyperbolicFunction):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * C.sin(i_coeff)
+                return S.ImaginaryUnit * sin(i_coeff)
             else:
                 if _coeff_isneg(arg):
                     return -cls(-arg)
 
+            if arg.is_Add:
+                x, m = _peeloff_ipi(arg)
+                if m:
+                    return sinh(m)*cosh(x) + cosh(m)*sinh(x)
+
             if arg.func == asinh:
                 return arg.args[0]
 
@@ -107,7 +157,7 @@ class sinh(HyperbolicFunction):
                 p = previous_terms[-2]
                 return p * x**2 / (n*(n - 1))
             else:
-                return x**(n) / C.factorial(n)
+                return x**(n) / factorial(n)
 
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
@@ -116,6 +166,7 @@ class sinh(HyperbolicFunction):
         """
         Returns this function as a complex coordinate.
         """
+        from sympy import cos, sin
         if self.args[0].is_real:
             if deep:
                 hints['complex'] = False
@@ -126,7 +177,7 @@ class sinh(HyperbolicFunction):
             re, im = self.args[0].expand(deep, **hints).as_real_imag()
         else:
             re, im = self.args[0].as_real_imag()
-        return (sinh(re)*C.cos(im), cosh(re)*C.sin(im))
+        return (sinh(re)*cos(im), cosh(re)*sin(im))
 
     def _eval_expand_complex(self, deep=True, **hints):
         re_part, im_part = self.as_real_imag(deep=deep, **hints)
@@ -150,10 +201,10 @@ class sinh(HyperbolicFunction):
         return sinh(arg)
 
     def _eval_rewrite_as_tractable(self, arg):
-        return (C.exp(arg) - C.exp(-arg)) / 2
+        return (exp(arg) - exp(-arg)) / 2
 
     def _eval_rewrite_as_exp(self, arg):
-        return (C.exp(arg) - C.exp(-arg)) / 2
+        return (exp(arg) - exp(-arg)) / 2
 
     def _eval_rewrite_as_cosh(self, arg):
         return -S.ImaginaryUnit*cosh(arg + S.Pi*S.ImaginaryUnit/2)
@@ -167,25 +218,31 @@ class sinh(HyperbolicFunction):
         return 2*coth_half/(coth_half**2 - 1)
 
     def _eval_as_leading_term(self, x):
+        from sympy import Order
         arg = self.args[0].as_leading_term(x)
 
-        if x in arg.free_symbols and C.Order(1, x).contains(arg):
-            return S.One
+        if x in arg.free_symbols and Order(1, x).contains(arg):
+            return arg
         else:
             return self.func(arg)
 
     def _eval_is_real(self):
-        return self.args[0].is_real
+        if self.args[0].is_real:
+            return True
+
+    def _eval_is_positive(self):
+        if self.args[0].is_real:
+            return self.args[0].is_positive
+
+    def _eval_is_negative(self):
+        if self.args[0].is_real:
+            return self.args[0].is_negative
 
     def _eval_is_finite(self):
         arg = self.args[0]
         if arg.is_imaginary:
             return True
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.sinh(self.args[0]._sage_())
-
 
 class cosh(HyperbolicFunction):
     r"""
@@ -207,6 +264,7 @@ class cosh(HyperbolicFunction):
 
     @classmethod
     def eval(cls, arg):
+        from sympy import cos
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -227,11 +285,16 @@ class cosh(HyperbolicFunction):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return C.cos(i_coeff)
+                return cos(i_coeff)
             else:
                 if _coeff_isneg(arg):
                     return cls(-arg)
 
+            if arg.is_Add:
+                x, m = _peeloff_ipi(arg)
+                if m:
+                    return cosh(m)*cosh(x) + sinh(m)*sinh(x)
+
             if arg.func == asinh:
                 return sqrt(1 + arg.args[0]**2)
 
@@ -257,12 +320,13 @@ class cosh(HyperbolicFunction):
                 p = previous_terms[-2]
                 return p * x**2 / (n*(n - 1))
             else:
-                return x**(n)/C.factorial(n)
+                return x**(n)/factorial(n)
 
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
 
     def as_real_imag(self, deep=True, **hints):
+        from sympy import cos, sin
         if self.args[0].is_real:
             if deep:
                 hints['complex'] = False
@@ -274,7 +338,7 @@ class cosh(HyperbolicFunction):
         else:
             re, im = self.args[0].as_real_imag()
 
-        return (cosh(re)*C.cos(im), sinh(re)*C.sin(im))
+        return (cosh(re)*cos(im), sinh(re)*sin(im))
 
     def _eval_expand_complex(self, deep=True, **hints):
         re_part, im_part = self.as_real_imag(deep=deep, **hints)
@@ -298,10 +362,10 @@ class cosh(HyperbolicFunction):
         return cosh(arg)
 
     def _eval_rewrite_as_tractable(self, arg):
-        return (C.exp(arg) + C.exp(-arg)) / 2
+        return (exp(arg) + exp(-arg)) / 2
 
     def _eval_rewrite_as_exp(self, arg):
-        return (C.exp(arg) + C.exp(-arg)) / 2
+        return (exp(arg) + exp(-arg)) / 2
 
     def _eval_rewrite_as_sinh(self, arg):
         return -S.ImaginaryUnit*sinh(arg + S.Pi*S.ImaginaryUnit/2)
@@ -315,25 +379,23 @@ class cosh(HyperbolicFunction):
         return (coth_half + 1)/(coth_half - 1)
 
     def _eval_as_leading_term(self, x):
+        from sympy import Order
         arg = self.args[0].as_leading_term(x)
 
-        if x in arg.free_symbols and C.Order(1, x).contains(arg):
+        if x in arg.free_symbols and Order(1, x).contains(arg):
             return S.One
         else:
             return self.func(arg)
 
-    def _eval_is_real(self):
-        return self.args[0].is_real
+    def _eval_is_positive(self):
+        if self.args[0].is_real:
+            return True
 
     def _eval_is_finite(self):
         arg = self.args[0]
         if arg.is_imaginary:
             return True
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.cosh(self.args[0]._sage_())
-
 
 class tanh(HyperbolicFunction):
     r"""
@@ -361,6 +423,7 @@ class tanh(HyperbolicFunction):
 
     @classmethod
     def eval(cls, arg):
+        from sympy import tan
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -382,12 +445,21 @@ class tanh(HyperbolicFunction):
 
             if i_coeff is not None:
                 if _coeff_isneg(i_coeff):
-                    return -S.ImaginaryUnit * C.tan(-i_coeff)
-                return S.ImaginaryUnit * C.tan(i_coeff)
+                    return -S.ImaginaryUnit * tan(-i_coeff)
+                return S.ImaginaryUnit * tan(i_coeff)
             else:
                 if _coeff_isneg(arg):
                     return -cls(-arg)
 
+            if arg.is_Add:
+                x, m = _peeloff_ipi(arg)
+                if m:
+                    tanhm = tanh(m)
+                    if tanhm is S.ComplexInfinity:
+                        return coth(x)
+                    else: # tanhm == 0
+                        return tanh(x)
+
             if arg.func == asinh:
                 x = arg.args[0]
                 return x/sqrt(1 + x**2)
@@ -405,6 +477,7 @@ class tanh(HyperbolicFunction):
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms):
+        from sympy import bernoulli
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
@@ -412,8 +485,8 @@ class tanh(HyperbolicFunction):
 
             a = 2**(n + 1)
 
-            B = C.bernoulli(n + 1)
-            F = C.factorial(n + 1)
+            B = bernoulli(n + 1)
+            F = factorial(n + 1)
 
             return a*(a - 1) * B/F * x**n
 
@@ -421,6 +494,7 @@ class tanh(HyperbolicFunction):
         return self.func(self.args[0].conjugate())
 
     def as_real_imag(self, deep=True, **hints):
+        from sympy import cos, sin
         if self.args[0].is_real:
             if deep:
                 hints['complex'] = False
@@ -431,15 +505,15 @@ class tanh(HyperbolicFunction):
             re, im = self.args[0].expand(deep, **hints).as_real_imag()
         else:
             re, im = self.args[0].as_real_imag()
-        denom = sinh(re)**2 + C.cos(im)**2
-        return (sinh(re)*cosh(re)/denom, C.sin(im)*C.cos(im)/denom)
+        denom = sinh(re)**2 + cos(im)**2
+        return (sinh(re)*cosh(re)/denom, sin(im)*cos(im)/denom)
 
     def _eval_rewrite_as_tractable(self, arg):
-        neg_exp, pos_exp = C.exp(-arg), C.exp(arg)
+        neg_exp, pos_exp = exp(-arg), exp(arg)
         return (pos_exp - neg_exp)/(pos_exp + neg_exp)
 
     def _eval_rewrite_as_exp(self, arg):
-        neg_exp, pos_exp = C.exp(-arg), C.exp(arg)
+        neg_exp, pos_exp = exp(-arg), exp(arg)
         return (pos_exp - neg_exp)/(pos_exp + neg_exp)
 
     def _eval_rewrite_as_sinh(self, arg):
@@ -452,25 +526,31 @@ class tanh(HyperbolicFunction):
         return 1/coth(arg)
 
     def _eval_as_leading_term(self, x):
+        from sympy import Order
         arg = self.args[0].as_leading_term(x)
 
-        if x in arg.free_symbols and C.Order(1, x).contains(arg):
-            return S.One
+        if x in arg.free_symbols and Order(1, x).contains(arg):
+            return arg
         else:
             return self.func(arg)
 
     def _eval_is_real(self):
-        return self.args[0].is_real
+        if self.args[0].is_real:
+            return True
+
+    def _eval_is_positive(self):
+        if self.args[0].is_real:
+            return self.args[0].is_positive
+
+    def _eval_is_negative(self):
+        if self.args[0].is_real:
+            return self.args[0].is_negative
 
     def _eval_is_finite(self):
         arg = self.args[0]
         if arg.is_real:
             return True
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.tanh(self.args[0]._sage_())
-
 
 class coth(HyperbolicFunction):
     r"""
@@ -493,6 +573,7 @@ class coth(HyperbolicFunction):
 
     @classmethod
     def eval(cls, arg):
+        from sympy import cot
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -514,12 +595,21 @@ class coth(HyperbolicFunction):
 
             if i_coeff is not None:
                 if _coeff_isneg(i_coeff):
-                    return S.ImaginaryUnit * C.cot(-i_coeff)
-                return -S.ImaginaryUnit * C.cot(i_coeff)
+                    return S.ImaginaryUnit * cot(-i_coeff)
+                return -S.ImaginaryUnit * cot(i_coeff)
             else:
                 if _coeff_isneg(arg):
                     return -cls(-arg)
 
+            if arg.is_Add:
+                x, m = _peeloff_ipi(arg)
+                if m:
+                    cothm = coth(m)
+                    if cothm is S.ComplexInfinity:
+                        return coth(x)
+                    else: # cothm == 0
+                        return tanh(x)
+
             if arg.func == asinh:
                 x = arg.args[0]
                 return sqrt(1 + x**2)/x
@@ -537,6 +627,7 @@ class coth(HyperbolicFunction):
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms):
+        from sympy import bernoulli
         if n == 0:
             return 1 / sympify(x)
         elif n < 0 or n % 2 == 0:
@@ -544,8 +635,8 @@ class coth(HyperbolicFunction):
         else:
             x = sympify(x)
 
-            B = C.bernoulli(n + 1)
-            F = C.factorial(n + 1)
+            B = bernoulli(n + 1)
+            F = factorial(n + 1)
 
             return 2**(n + 1) * B/F * x**n
 
@@ -553,6 +644,7 @@ class coth(HyperbolicFunction):
         return self.func(self.args[0].conjugate())
 
     def as_real_imag(self, deep=True, **hints):
+        from sympy import cos, sin
         if self.args[0].is_real:
             if deep:
                 hints['complex'] = False
@@ -563,15 +655,15 @@ class coth(HyperbolicFunction):
             re, im = self.args[0].expand(deep, **hints).as_real_imag()
         else:
             re, im = self.args[0].as_real_imag()
-        denom = sinh(re)**2 + C.sin(im)**2
-        return (sinh(re)*cosh(re)/denom, -C.sin(im)*C.cos(im)/denom)
+        denom = sinh(re)**2 + sin(im)**2
+        return (sinh(re)*cosh(re)/denom, -sin(im)*cos(im)/denom)
 
     def _eval_rewrite_as_tractable(self, arg):
-        neg_exp, pos_exp = C.exp(-arg), C.exp(arg)
+        neg_exp, pos_exp = exp(-arg), exp(arg)
         return (pos_exp + neg_exp)/(pos_exp - neg_exp)
 
     def _eval_rewrite_as_exp(self, arg):
-        neg_exp, pos_exp = C.exp(-arg), C.exp(arg)
+        neg_exp, pos_exp = exp(-arg), exp(arg)
         return (pos_exp + neg_exp)/(pos_exp - neg_exp)
 
     def _eval_rewrite_as_sinh(self, arg):
@@ -583,24 +675,209 @@ class coth(HyperbolicFunction):
     def _eval_rewrite_as_tanh(self, arg):
         return 1/tanh(arg)
 
+    def _eval_is_positive(self):
+        if self.args[0].is_real:
+            return self.args[0].is_positive
+
+    def _eval_is_negative(self):
+        if self.args[0].is_real:
+            return self.args[0].is_negative
+
     def _eval_as_leading_term(self, x):
+        from sympy import Order
         arg = self.args[0].as_leading_term(x)
 
-        if x in arg.free_symbols and C.Order(1, x).contains(arg):
-            return S.One
+        if x in arg.free_symbols and Order(1, x).contains(arg):
+            return 1/arg
         else:
             return self.func(arg)
 
+
+class ReciprocalHyperbolicFunction(HyperbolicFunction):
+    """Base class for reciprocal functions of hyperbolic functions. """
+
+    #To be defined in class
+    _reciprocal_of = None
+    _is_even = None
+    _is_odd = None
+
+    @classmethod
+    def eval(cls, arg):
+        if arg.could_extract_minus_sign():
+            if cls._is_even:
+                return cls(-arg)
+            if cls._is_odd:
+                return -cls(-arg)
+
+        t = cls._reciprocal_of.eval(arg)
+        if hasattr(arg, 'inverse') and arg.inverse() == cls:
+            return arg.args[0]
+        return 1/t if t != None else t
+
+    def _call_reciprocal(self, method_name, *args, **kwargs):
+        # Calls method_name on _reciprocal_of
+        o = self._reciprocal_of(self.args[0])
+        return getattr(o, method_name)(*args, **kwargs)
+
+    def _calculate_reciprocal(self, method_name, *args, **kwargs):
+        # If calling method_name on _reciprocal_of returns a value != None
+        # then return the reciprocal of that value
+        t = self._call_reciprocal(method_name, *args, **kwargs)
+        return 1/t if t != None else t
+
+    def _rewrite_reciprocal(self, method_name, arg):
+        # Special handling for rewrite functions. If reciprocal rewrite returns
+        # unmodified expression, then return None
+        t = self._call_reciprocal(method_name, arg)
+        if t != None and t != self._reciprocal_of(arg):
+            return 1/t
+
+    def _eval_rewrite_as_exp(self, arg):
+        return self._rewrite_reciprocal("_eval_rewrite_as_exp", arg)
+
+    def _eval_rewrite_as_tractable(self, arg):
+        return self._rewrite_reciprocal("_eval_rewrite_as_tractable", arg)
+
+    def _eval_rewrite_as_tanh(self, arg):
+        return self._rewrite_reciprocal("_eval_rewrite_as_tanh", arg)
+
+    def _eval_rewrite_as_coth(self, arg):
+        return self._rewrite_reciprocal("_eval_rewrite_as_coth", arg)
+
+    def as_real_imag(self, deep = True, **hints):
+        return (1 / self._reciprocal_of(self.args[0])).as_real_imag(deep, **hints)
+
+    def _eval_conjugate(self):
+        return self.func(self.args[0].conjugate())
+
+    def _eval_expand_complex(self, deep=True, **hints):
+        re_part, im_part = self.as_real_imag(deep=True, **hints)
+        return re_part + S.ImaginaryUnit*im_part
+
+    def _eval_as_leading_term(self, x):
+        return (1/self._reciprocal_of(self.args[0]))._eval_as_leading_term(x)
+
+    def _eval_is_real(self):
+        return self._reciprocal_of(self.args[0]).is_real
+
+    def _eval_is_finite(self):
+        return (1/self._reciprocal_of(self.args[0])).is_finite
+
+
+class csch(ReciprocalHyperbolicFunction):
+    r"""
+    The hyperbolic cosecant function, `\frac{2}{e^x - e^{-x}}`
+
+    * csch(x) -> Returns the hyperbolic cosecant of x
+
+    See Also
+    ========
+
+    sinh, cosh, tanh, sech, asinh, acosh
+    """
+
+    _reciprocal_of = sinh
+    _is_odd = True
+
+    def fdiff(self, argindex=1):
+        """
+        Returns the first derivative of this function
+        """
+        if argindex == 1:
+            return -coth(self.args[0]) * csch(self.args[0])
+        else:
+            raise ArgumentIndexError(self, argindex)
+
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
+        """
+        Returns the next term in the Taylor series expansion
+        """
+        from sympy import bernoulli
+        if n == 0:
+            return 1/sympify(x)
+        elif n < 0 or n % 2 == 0:
+            return S.Zero
+        else:
+            x = sympify(x)
+
+            B = bernoulli(n + 1)
+            F = factorial(n + 1)
+
+            return 2 * (1 - 2**n) * B/F * x**n
+
+    def _eval_rewrite_as_cosh(self, arg):
+        return S.ImaginaryUnit / cosh(arg + S.ImaginaryUnit * S.Pi / 2)
+
+    def _eval_is_positive(self):
+        if self.args[0].is_real:
+            return self.args[0].is_positive
+
+    def _eval_is_negative(self):
+        if self.args[0].is_real:
+            return self.args[0].is_negative
+
     def _sage_(self):
         import sage.all as sage
-        return sage.coth(self.args[0]._sage_())
+        return sage.csch(self.args[0]._sage_())
+
+
+class sech(ReciprocalHyperbolicFunction):
+    r"""
+    The hyperbolic secant function, `\frac{2}{e^x + e^{-x}}`
+
+    * sech(x) -> Returns the hyperbolic secant of x
+
+    See Also
+    ========
+
+    sinh, cosh, tanh, coth, csch, asinh, acosh
+    """
+
+    _reciprocal_of = cosh
+    _is_even = True
+
+    def fdiff(self, argindex=1):
+        if argindex == 1:
+            return - tanh(self.args[0])*sech(self.args[0])
+        else:
+            raise ArgumentIndexError(self, argindex)
+
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
+        from sympy.functions.combinatorial.numbers import euler
+        if n < 0 or n % 2 == 1:
+            return S.Zero
+        else:
+            x = sympify(x)
+            return euler(n) / factorial(n) * x**(n)
+
+    def _eval_rewrite_as_sinh(self, arg):
+        return S.ImaginaryUnit / sinh(arg + S.ImaginaryUnit * S.Pi /2)
+
+    def _eval_is_positive(self):
+        if self.args[0].is_real:
+            return True
+
+    def _sage_(self):
+        import sage.all as sage
+        return sage.sech(self.args[0]._sage_())
+
 
 
 ###############################################################################
 ############################# HYPERBOLIC INVERSES #############################
 ###############################################################################
 
-class asinh(Function):
+class InverseHyperbolicFunction(Function):
+    """Base class for inverse hyperbolic functions."""
+
+    pass
+
+
+class asinh(InverseHyperbolicFunction):
     """
     The inverse hyperbolic sine function.
 
@@ -620,6 +897,7 @@ class asinh(Function):
 
     @classmethod
     def eval(cls, arg):
+        from sympy import asin
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -632,9 +910,9 @@ class asinh(Function):
             elif arg is S.Zero:
                 return S.Zero
             elif arg is S.One:
-                return C.log(sqrt(2) + 1)
+                return log(sqrt(2) + 1)
             elif arg is S.NegativeOne:
-                return C.log(sqrt(2) - 1)
+                return log(sqrt(2) - 1)
             elif arg.is_negative:
                 return -cls(-arg)
         else:
@@ -644,7 +922,7 @@ class asinh(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * C.asin(i_coeff)
+                return S.ImaginaryUnit * asin(i_coeff)
             else:
                 if _coeff_isneg(arg):
                     return -cls(-arg)
@@ -661,30 +939,30 @@ class asinh(Function):
                 return -p * (n - 2)**2/(n*(n - 1)) * x**2
             else:
                 k = (n - 1) // 2
-                R = C.RisingFactorial(S.Half, k)
-                F = C.factorial(k)
+                R = RisingFactorial(S.Half, k)
+                F = factorial(k)
                 return (-1)**k * R / F * x**n / n
 
     def _eval_as_leading_term(self, x):
+        from sympy import Order
         arg = self.args[0].as_leading_term(x)
 
-        if x in arg.free_symbols and C.Order(1, x).contains(arg):
+        if x in arg.free_symbols and Order(1, x).contains(arg):
             return arg
         else:
             return self.func(arg)
 
+    def _eval_rewrite_as_log(self, x):
+        return log(x + sqrt(x**2 + 1))
+
     def inverse(self, argindex=1):
         """
         Returns the inverse of this function.
         """
         return sinh
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.asinh(self.args[0]._sage_())
 
-
-class acosh(Function):
+class acosh(InverseHyperbolicFunction):
     """
     The inverse hyperbolic cosine function.
 
@@ -722,8 +1000,8 @@ class acosh(Function):
 
         if arg.is_number:
             cst_table = {
-                S.ImaginaryUnit: C.log(S.ImaginaryUnit*(1 + sqrt(2))),
-                -S.ImaginaryUnit: C.log(-S.ImaginaryUnit*(1 + sqrt(2))),
+                S.ImaginaryUnit: log(S.ImaginaryUnit*(1 + sqrt(2))),
+                -S.ImaginaryUnit: log(-S.ImaginaryUnit*(1 + sqrt(2))),
                 S.Half: S.Pi/3,
                 -S.Half: 2*S.Pi/3,
                 sqrt(2)/2: S.Pi/4,
@@ -750,17 +1028,11 @@ class acosh(Function):
                 return cst_table[arg]
 
         if arg is S.ComplexInfinity:
-            return S.Infinity
-
-        i_coeff = arg.as_coefficient(S.ImaginaryUnit)
-
-        if i_coeff is not None:
-            if _coeff_isneg(i_coeff):
-                return S.ImaginaryUnit * C.acos(i_coeff)
-            return S.ImaginaryUnit * C.acos(-i_coeff)
-        else:
-            if _coeff_isneg(arg):
-                return -cls(-arg)
+            return S.ComplexInfinity
+        if arg == S.ImaginaryUnit*S.Infinity:
+            return S.Infinity + S.ImaginaryUnit*S.Pi/2
+        if arg == -S.ImaginaryUnit*S.Infinity:
+            return S.Infinity - S.ImaginaryUnit*S.Pi/2
 
     @staticmethod
     @cacheit
@@ -776,30 +1048,30 @@ class acosh(Function):
                 return p * (n - 2)**2/(n*(n - 1)) * x**2
             else:
                 k = (n - 1) // 2
-                R = C.RisingFactorial(S.Half, k)
-                F = C.factorial(k)
+                R = RisingFactorial(S.Half, k)
+                F = factorial(k)
                 return -R / F * S.ImaginaryUnit * x**n / n
 
     def _eval_as_leading_term(self, x):
+        from sympy import Order
         arg = self.args[0].as_leading_term(x)
 
-        if x in arg.free_symbols and C.Order(1, x).contains(arg):
-            return arg
+        if x in arg.free_symbols and Order(1, x).contains(arg):
+            return S.ImaginaryUnit*S.Pi/2
         else:
             return self.func(arg)
 
+    def _eval_rewrite_as_log(self, x):
+        return log(x + sqrt(x + 1) * sqrt(x - 1))
+
     def inverse(self, argindex=1):
         """
         Returns the inverse of this function.
         """
         return cosh
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.acosh(self.args[0]._sage_())
 
-
-class atanh(Function):
+class atanh(InverseHyperbolicFunction):
     """
     The inverse hyperbolic tangent function.
 
@@ -819,6 +1091,7 @@ class atanh(Function):
 
     @classmethod
     def eval(cls, arg):
+        from sympy import atan
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -831,19 +1104,20 @@ class atanh(Function):
             elif arg is S.NegativeOne:
                 return S.NegativeInfinity
             elif arg is S.Infinity:
-                return -S.ImaginaryUnit * C.atan(arg)
+                return -S.ImaginaryUnit * atan(arg)
             elif arg is S.NegativeInfinity:
-                return S.ImaginaryUnit * C.atan(-arg)
+                return S.ImaginaryUnit * atan(-arg)
             elif arg.is_negative:
                 return -cls(-arg)
         else:
             if arg is S.ComplexInfinity:
-                return S.NaN
+                from sympy.calculus.util import AccumBounds
+                return S.ImaginaryUnit*AccumBounds(-S.Pi/2, S.Pi/2)
 
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * C.atan(i_coeff)
+                return S.ImaginaryUnit * atan(i_coeff)
             else:
                 if _coeff_isneg(arg):
                     return -cls(-arg)
@@ -858,25 +1132,25 @@ class atanh(Function):
             return x**n / n
 
     def _eval_as_leading_term(self, x):
+        from sympy import Order
         arg = self.args[0].as_leading_term(x)
 
-        if x in arg.free_symbols and C.Order(1, x).contains(arg):
+        if x in arg.free_symbols and Order(1, x).contains(arg):
             return arg
         else:
             return self.func(arg)
 
+    def _eval_rewrite_as_log(self, x):
+        return (log(1 + x) - log(1 - x)) / 2
+
     def inverse(self, argindex=1):
         """
         Returns the inverse of this function.
         """
         return tanh
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.atanh(self.args[0]._sage_())
 
-
-class acoth(Function):
+class acoth(InverseHyperbolicFunction):
     """
     The inverse hyperbolic cotangent function.
 
@@ -891,6 +1165,7 @@ class acoth(Function):
 
     @classmethod
     def eval(cls, arg):
+        from sympy import acot
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -910,12 +1185,12 @@ class acoth(Function):
                 return -cls(-arg)
         else:
             if arg is S.ComplexInfinity:
-                return 0
+                return S.Zero
 
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return -S.ImaginaryUnit * C.acot(i_coeff)
+                return -S.ImaginaryUnit * acot(i_coeff)
             else:
                 if _coeff_isneg(arg):
                     return -cls(-arg)
@@ -932,19 +1207,241 @@ class acoth(Function):
             return x**n / n
 
     def _eval_as_leading_term(self, x):
+        from sympy import Order
         arg = self.args[0].as_leading_term(x)
 
-        if x in arg.free_symbols and C.Order(1, x).contains(arg):
-            return arg
+        if x in arg.free_symbols and Order(1, x).contains(arg):
+            return S.ImaginaryUnit*S.Pi/2
         else:
             return self.func(arg)
 
+    def _eval_rewrite_as_log(self, x):
+        return (log(1 + 1/x) - log(1 - 1/x)) / 2
+
     def inverse(self, argindex=1):
         """
         Returns the inverse of this function.
         """
         return coth
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.acoth(self.args[0]._sage_())
+
+class asech(InverseHyperbolicFunction):
+    """
+    The inverse hyperbolic secant function.
+
+    * asech(x) -> Returns the inverse hyperbolic secant of x
+
+    Examples
+    ========
+
+    >>> from sympy import asech, sqrt, S
+    >>> from sympy.abc import x
+    >>> asech(x).diff(x)
+    -1/(x*sqrt(-x**2 + 1))
+    >>> asech(1).diff(x)
+    0
+    >>> asech(1)
+    0
+    >>> asech(S(2))
+    I*pi/3
+    >>> asech(-sqrt(2))
+    3*I*pi/4
+    >>> asech((sqrt(6) - sqrt(2)))
+    I*pi/12
+
+    See Also
+    ========
+
+    asinh, atanh, cosh, acoth
+
+    References
+    ==========
+
+    .. [1] http://en.wikipedia.org/wiki/Hyperbolic_function
+    .. [2] http://dlmf.nist.gov/4.37
+    .. [3] http://functions.wolfram.com/ElementaryFunctions/ArcSech/
+
+    """
+
+    def fdiff(self, argindex=1):
+        if argindex == 1:
+            z = self.args[0]
+            return -1/(z*sqrt(1 - z**2))
+        else:
+            raise ArgumentIndexError(self, argindex)
+
+    @classmethod
+    def eval(cls, arg):
+        arg = sympify(arg)
+
+        if arg.is_Number:
+            if arg is S.NaN:
+                return S.NaN
+            elif arg is S.Infinity:
+                return S.Pi*S.ImaginaryUnit / 2
+            elif arg is S.NegativeInfinity:
+                return S.Pi*S.ImaginaryUnit / 2
+            elif arg is S.Zero:
+                return S.Infinity
+            elif arg is S.One:
+                return S.Zero
+            elif arg is S.NegativeOne:
+                return S.Pi*S.ImaginaryUnit
+
+        if arg.is_number:
+            cst_table = {
+                S.ImaginaryUnit: - (S.Pi*S.ImaginaryUnit / 2) + log(1 + sqrt(2)),
+                -S.ImaginaryUnit: (S.Pi*S.ImaginaryUnit / 2) + log(1 + sqrt(2)),
+                (sqrt(6) - sqrt(2)): S.Pi / 12,
+                (sqrt(2) - sqrt(6)): 11*S.Pi / 12,
+                sqrt(2 - 2/sqrt(5)): S.Pi / 10,
+                -sqrt(2 - 2/sqrt(5)): 9*S.Pi / 10,
+                2 / sqrt(2 + sqrt(2)): S.Pi / 8,
+                -2 / sqrt(2 + sqrt(2)): 7*S.Pi / 8,
+                2 / sqrt(3): S.Pi / 6,
+                -2 / sqrt(3): 5*S.Pi / 6,
+                (sqrt(5) - 1): S.Pi / 5,
+                (1 - sqrt(5)): 4*S.Pi / 5,
+                sqrt(2): S.Pi / 4,
+                -sqrt(2): 3*S.Pi / 4,
+                sqrt(2 + 2/sqrt(5)): 3*S.Pi / 10,
+                -sqrt(2 + 2/sqrt(5)): 7*S.Pi / 10,
+                S(2): S.Pi / 3,
+                -S(2): 2*S.Pi / 3,
+                sqrt(2*(2 + sqrt(2))): 3*S.Pi / 8,
+                -sqrt(2*(2 + sqrt(2))): 5*S.Pi / 8,
+                (1 + sqrt(5)): 2*S.Pi / 5,
+                (-1 - sqrt(5)): 3*S.Pi / 5,
+                (sqrt(6) + sqrt(2)): 5*S.Pi / 12,
+                (-sqrt(6) - sqrt(2)): 7*S.Pi / 12,
+            }
+
+            if arg in cst_table:
+                if arg.is_real:
+                    return cst_table[arg]*S.ImaginaryUnit
+                return cst_table[arg]
+
+        if arg is S.ComplexInfinity:
+            from sympy.calculus.util import AccumBounds
+            return S.ImaginaryUnit*AccumBounds(-S.Pi/2, S.Pi/2)
+
+    @staticmethod
+    @cacheit
+    def expansion_term(n, x, *previous_terms):
+        if n == 0:
+            return log(2 / x)
+        elif n < 0 or n % 2 == 1:
+            return S.Zero
+        else:
+            x = sympify(x)
+            if len(previous_terms) > 2 and n > 2:
+                p = previous_terms[-2]
+                return p * (n - 1)**2 // (n // 2)**2 * x**2 / 4
+            else:
+                k = n // 2
+                R = RisingFactorial(S.Half , k) *  n
+                F = factorial(k) * n // 2 * n // 2
+                return -1 * R / F * x**n / 4
+
+    def inverse(self, argindex=1):
+        """
+        Returns the inverse of this function.
+        """
+        return sech
+
+    def _eval_rewrite_as_log(self, arg):
+        return log(1/arg + sqrt(1/arg - 1) * sqrt(1/arg + 1))
+
+
+class acsch(InverseHyperbolicFunction):
+    """
+    The inverse hyperbolic cosecant function.
+
+    * acsch(x) -> Returns the inverse hyperbolic cosecant of x
+
+    Examples
+    ========
+
+    >>> from sympy import acsch, sqrt, S
+    >>> from sympy.abc import x
+    >>> acsch(x).diff(x)
+    -1/(x**2*sqrt(1 + x**(-2)))
+    >>> acsch(1).diff(x)
+    0
+    >>> acsch(1)
+    log(1 + sqrt(2))
+    >>> acsch(S.ImaginaryUnit)
+    -I*pi/2
+    >>> acsch(-2*S.ImaginaryUnit)
+    I*pi/6
+    >>> acsch(S.ImaginaryUnit*(sqrt(6) - sqrt(2)))
+    -5*I*pi/12
+
+    References
+    ==========
+
+    .. [1] http://en.wikipedia.org/wiki/Hyperbolic_function
+    .. [2] http://dlmf.nist.gov/4.37
+    .. [3] http://functions.wolfram.com/ElementaryFunctions/ArcCsch/
+
+    """
+
+    def fdiff(self, argindex=1):
+        if argindex == 1:
+            z = self.args[0]
+            return -1/(z**2*sqrt(1 + 1/z**2))
+        else:
+            raise ArgumentIndexError(self, argindex)
+
+    @classmethod
+    def eval(cls, arg):
+        arg = sympify(arg)
+
+        if arg.is_Number:
+            if arg is S.NaN:
+                return S.NaN
+            elif arg is S.Infinity:
+                return S.Zero
+            elif arg is S.NegativeInfinity:
+                return S.Zero
+            elif arg is S.Zero:
+                return S.ComplexInfinity
+            elif arg is S.One:
+                return log(1 + sqrt(2))
+            elif arg is S.NegativeOne:
+                return - log(1 + sqrt(2))
+
+        if arg.is_number:
+            cst_table = {
+                S.ImaginaryUnit: -S.Pi / 2,
+                S.ImaginaryUnit*(sqrt(2) + sqrt(6)): -S.Pi / 12,
+                S.ImaginaryUnit*(1 + sqrt(5)): -S.Pi / 10,
+                S.ImaginaryUnit*2 / sqrt(2 - sqrt(2)): -S.Pi / 8,
+                S.ImaginaryUnit*2: -S.Pi / 6,
+                S.ImaginaryUnit*sqrt(2 + 2/sqrt(5)): -S.Pi / 5,
+                S.ImaginaryUnit*sqrt(2): -S.Pi / 4,
+                S.ImaginaryUnit*(sqrt(5)-1): -3*S.Pi / 10,
+                S.ImaginaryUnit*2 / sqrt(3): -S.Pi / 3,
+                S.ImaginaryUnit*2 / sqrt(2 + sqrt(2)): -3*S.Pi / 8,
+                S.ImaginaryUnit*sqrt(2 - 2/sqrt(5)): -2*S.Pi / 5,
+                S.ImaginaryUnit*(sqrt(6) - sqrt(2)): -5*S.Pi / 12,
+                S(2): -S.ImaginaryUnit*log((1+sqrt(5))/2),
+            }
+
+            if arg in cst_table:
+                return cst_table[arg]*S.ImaginaryUnit
+
+        if arg is S.ComplexInfinity:
+            return S.Zero
+
+        if _coeff_isneg(arg):
+            return -cls(-arg)
+
+    def inverse(self, argindex=1):
+        """
+        Returns the inverse of this function.
+        """
+        return csch
+
+    def _eval_rewrite_as_log(self, arg):
+        return log(1/arg + sqrt(1/arg**2 + 1))
