@@ -11,14 +11,15 @@ from sympy.polys.specialpolys import (
 
 from sympy.polys.polyerrors import (
     PolificationFailed, ComputationFailed,
-    MultivariatePolynomialError)
+    MultivariatePolynomialError, OptionError)
 
 from sympy.utilities import numbered_symbols, take, public
 
-from sympy.core import S, Basic, Add, Mul
+from sympy.core import S, Basic, Add, Mul, symbols
 
-from sympy.core.compatibility import xrange
+from sympy.core.compatibility import range
 
+from sympy.functions.combinatorial.factorials import factorial
 
 @public
 def symmetrize(F, *gens, **args):
@@ -86,11 +87,11 @@ def symmetrize(F, *gens, **args):
     polys, symbols = [], opt.symbols
     gens, dom = opt.gens, opt.domain
 
-    for i in xrange(0, len(gens)):
+    for i in range(len(gens)):
         poly = symmetric_poly(i + 1, gens, polys=True)
         polys.append((next(symbols), poly.set_domain(dom)))
 
-    indices = list(range(0, len(gens) - 1))
+    indices = list(range(len(gens) - 1))
     weights = list(range(len(gens), 0, -1))
 
     result = []
@@ -107,7 +108,7 @@ def symmetrize(F, *gens, **args):
 
             for i, (monom, coeff) in enumerate(f.terms()):
                 if all(monom[i] >= monom[i + 1] for i in indices):
-                    height = max([ n*m for n, m in zip(weights, monom) ])
+                    height = max([n*m for n, m in zip(weights, monom)])
 
                     if height > _height:
                         _height, _monom, _coeff = height, monom, coeff
@@ -122,8 +123,8 @@ def symmetrize(F, *gens, **args):
             for m1, m2 in zip(monom, monom[1:] + (0,)):
                 exponents.append(m1 - m2)
 
-            term = [ s**n for (s, _), n in zip(polys, exponents) ]
-            poly = [ p**n for (_, p), n in zip(polys, exponents) ]
+            term = [s**n for (s, _), n in zip(polys, exponents)]
+            poly = [p**n for (_, p), n in zip(polys, exponents)]
 
             symmetric.append(Mul(coeff, *term))
             product = poly[0].mul(coeff)
@@ -135,7 +136,7 @@ def symmetrize(F, *gens, **args):
 
         result.append((Add(*symmetric), f.as_expr()))
 
-    polys = [ (s, p.as_expr()) for s, p in polys ]
+    polys = [(s, p.as_expr()) for s, p in polys]
 
     if not opt.formal:
         for i, (sym, non_sym) in enumerate(result):
@@ -239,21 +240,88 @@ def interpolate(data, x):
 
     """
     n = len(data)
+    poly = None
 
     if isinstance(data, dict):
         X, Y = list(zip(*data.items()))
+        poly = interpolating_poly(n, x, X, Y)
     else:
         if isinstance(data[0], tuple):
             X, Y = list(zip(*data))
+            poly = interpolating_poly(n, x, X, Y)
         else:
-            X = list(range(1, n + 1))
             Y = list(data)
 
-    poly = interpolating_poly(n, x, X, Y)
+            numert = Mul(*[(x - i) for i in range(1, n + 1)])
+            denom = -factorial(n - 1) if n%2 == 0 else factorial(n - 1)
+            coeffs = []
+            for i in range(1, n + 1):
+                coeffs.append(numert/(x - i)/denom)
+                denom = denom/(i - n)*i
+
+            poly = Add(*[coeff*y for coeff, y in zip(coeffs, Y)])
 
     return poly.expand()
 
 
+@public
+def rational_interpolate(data, degnum, X=symbols('x')):
+    """
+    Returns a rational interpolation, where the data points are element of
+    any integral domain.
+
+    The first argument  contains the data (as a list of coordinates). The
+    ``degnum`` argument is the degree in the numerator of the rational
+    function. Setting it too high will decrease the maximal degree in the
+    denominator for the same amount of data.
+
+    Example:
+    ========
+    >>> from sympy.polys.polyfuncs import rational_interpolate
+
+    >>> data = [(1, -210), (2, -35), (3, 105), (4, 231), (5, 350), (6, 465)]
+    >>> rational_interpolate(data, 2)
+    (105*x**2 - 525)/(x + 1)
+
+    Values do not need to be integers:
+
+    >>> from sympy import sympify
+    >>> x = [1, 2, 3, 4, 5, 6]
+    >>> y = sympify("[-1, 0, 2, 22/5, 7, 68/7]")
+    >>> rational_interpolate(zip(x, y), 2)
+    (3*x**2 - 7*x + 2)/(x + 1)
+
+    The symbol for the variable can be changed if needed:
+    >>> from sympy import symbols
+    >>> z = symbols('z')
+    >>> rational_interpolate(data, 2, X=z)
+    (105*z**2 - 525)/(z + 1)
+
+    References
+    ==========
+    Algorithm is adapted from:
+        http://axiom-wiki.newsynthesis.org/RationalInterpolation
+
+    """
+    from sympy.matrices.dense import ones
+
+    xdata, ydata = list(zip(*data))
+
+    k = len(xdata) - degnum - 1
+    if k < 0:
+        raise OptionError("Too few values for the required degree.")
+    c = ones(degnum + k + 1, degnum + k + 2)
+    for j in range(max(degnum, k)):
+        for i in range(degnum + k + 1):
+            c[i, j + 1] = c[i, j]*xdata[i]
+    for j in range(k + 1):
+        for i in range(degnum + k + 1):
+            c[i, degnum + k + 1 - j] = -c[i, k - j]*ydata[i]
+    r = c.nullspace()[0]
+    return (sum(r[i] * X**i for i in range(degnum + 1))
+            / sum(r[i + degnum + 1] * X**i for i in range(k + 1)))
+
+
 @public
 def viete(f, roots=None, *gens, **args):
     """
